unit KeyTable;

interface

uses Windows, ConfFile;

type

TCmd = record
  Name: string;
  Caption: string;
  Id: Integer;
end;

PCmdTable = ^TCmdTable;
TCmdTable = array[0..0] of TCmd;

TKeyTable = class(TObject)
private
  FKeyList: array[0..255] of TAccel;
  FKeyListCount: Integer;
  FCmdTable: PCmdTable;
  FCmdTableCount: Integer;
  function StrToAccel(const KeyName: string; const ActionName: string): TAccel;
  function AccelToStr(const Accel: TAccel): string;
  function IdToKeyNames(Id: Integer): string;
  procedure LoadFromFile(const FileName: string);
  procedure SetDefaultKeys(const DefaultKeys: array of string);
public
  AccHandle: HACCEL;
  MenuHandle: HMENU;
  constructor Create(const CmdTable: array of TCmd; const DefaultKeys: array of string;
    FileName: string);
  destructor Destroy; override;
  procedure AddMenuItem(Id: Integer);
  procedure GenerateMenu(const MenuTable: array of Integer);
end;

procedure StrToKey(const KeyName: string; var Alt, Control, Shift: Boolean; var VKey: UINT);

implementation

const
  SpKeys: array[0..41] of TCmd = (
    (Name: 'BS'; Id: VK_BACK),
    (Name: 'Tab'; Id: VK_TAB),
    (Name: 'Enter'; Id: VK_RETURN),
    (Name: 'Esc'; Id: VK_ESCAPE),
    (Name: 'Space'; Id: VK_SPACE),
    (Name: 'PgUp'; Id: VK_PRIOR),
    (Name: 'PgDn'; Id: VK_NEXT),
    (Name: 'End'; Id: VK_END),
    (Name: 'Home'; Id: VK_HOME),
    (Name: 'Left'; Id: VK_LEFT),
    (Name: 'Up'; Id: VK_UP),
    (Name: 'Right'; Id: VK_RIGHT),
    (Name: 'Down'; Id: VK_DOWN),
    (Name: 'Ins'; Id: VK_INSERT),
    (Name: 'Del'; Id: VK_DELETE),
    (Name: 'Num0'; Id: VK_NUMPAD0),
    (Name: 'Num1'; Id: VK_NUMPAD1),
    (Name: 'Num2'; Id: VK_NUMPAD2),
    (Name: 'Num3'; Id: VK_NUMPAD3),
    (Name: 'Num4'; Id: VK_NUMPAD4),
    (Name: 'Num5'; Id: VK_NUMPAD5),
    (Name: 'Num6'; Id: VK_NUMPAD6),
    (Name: 'Num7'; Id: VK_NUMPAD7),
    (Name: 'Num8'; Id: VK_NUMPAD8),
    (Name: 'Num9'; Id: VK_NUMPAD9),
    (Name: 'Num*'; Id: VK_MULTIPLY),
    (Name: 'Num+'; Id: VK_ADD),
    (Name: 'Num-'; Id: VK_SUBTRACT),
    (Name: 'Num.'; Id: VK_DECIMAL),
    (Name: 'Num/'; Id: VK_DIVIDE),
    (Name: 'F1'; Id: VK_F1),
    (Name: 'F2'; Id: VK_F2),
    (Name: 'F3'; Id: VK_F3),
    (Name: 'F4'; Id: VK_F4),
    (Name: 'F5'; Id: VK_F5),
    (Name: 'F6'; Id: VK_F6),
    (Name: 'F7'; Id: VK_F7),
    (Name: 'F8'; Id: VK_F8),
    (Name: 'F9'; Id: VK_F9),
    (Name: 'F10'; Id: VK_F10),
    (Name: 'F11'; Id: VK_F11),
    (Name: 'F12'; Id: VK_F12)
  );

procedure StrToKey(const KeyName: string; var Alt, Control, Shift: Boolean; var VKey: UINT);
var
  k, s: string;
  i: Integer;
  p: PChar;
begin
  Alt := False;
  Control := False;
  Shift := False;
  VKey := 0;
  if Length(KeyName) = 1 then
    k := KeyName
  else begin
    s := KeyName;
    i := Pos('-', s);
    if i = 0 then
      k := KeyName
    else begin
      s[i] := #0;
      k := PChar(@s[i+1]);
      p := PChar(s);
      Alt := Pos('A', p) > 0;
      Control := Pos('C', p) > 0;
      Shift := Pos('S', p) > 0;
    end;
  end;

  for i := 0 to High(SpKeys) do
  begin
    if k = SpKeys[i].Name then
    begin
      VKey := SpKeys[i].Id;
      Break;
    end;
  end;
  if VKey = 0 then
    VKey := Ord(k[1]);
end;

constructor TKeyTable.Create(const CmdTable: array of TCmd; const DefaultKeys: array of string;
  FileName: string);
begin
  FCmdTable := @CmdTable;
  FCmdTableCount := High(CmdTable) + 1;
  SetDefaultKeys(DefaultKeys);
  LoadFromFile(FileName);
  AccHandle := CreateAcceleratorTable(FKeyList, FKeyListCount);
end;

destructor TKeyTable.Destroy;
begin
  DestroyAcceleratorTable(AccHandle);
  inherited Destroy;
end;

function TKeyTable.IdToKeyNames(Id: Integer): string;
var
  i, j: Integer;
  s: string;
begin
  Result := '';
  for i := 0 to FCmdTableCount - 1 do
  begin
    if FCmdTable[i].Id = Id then
    begin
      for j := 0 to FKeyListCount - 1 do
      begin
        if FCmdTable[i].Id = FKeyList[j].cmd then
        begin
          if s = '' then
            s := AccelToStr(FKeyList[j])
          else
            s := s + ', ' + AccelToStr(FKeyList[j]);
        end;
      end;
      Result := FCmdTable[i].Caption + #9 + s;
      Exit;
    end;
  end;
end;

procedure TKeyTable.LoadFromFile(const FileName: string);
var
  ConfFile: TConfFile;
  i: Integer;
begin
  ConfFile := TConfFile.Create(FileName);
  i := 0;
  while ConfFile.GetLine do
  begin
    Inc(i);
    FKeyList[i-1] := StrToAccel(ConfFile.Line.Name, ConfFile.Line.Args[0]);
    if (i-1) = High(FKeyList) then
      Break;
  end;
  ConfFile.Free;
  if i > 0 then
    FKeyListCount := i;
end;

function TKeyTable.StrToAccel(const KeyName: string; const ActionName: string): TAccel;
var
  i: Integer;
  Alt, Control, Shift: Boolean;
  VKey: UINT;
begin
  Result.key := 0;
  Result.cmd := 0;
  Result.fVirt := FVIRTKEY;

  StrToKey(KeyName, Alt, Control, Shift, VKey);
  Result.key := VKey;
  if Alt then Result.fVirt := Result.fVirt or FALT;
  if Control then Result.fVirt := Result.fVirt or FCONTROL;
  if Shift then Result.fVirt := Result.fVirt or FSHIFT;

  for i := 0 to FCmdTableCount - 1 do
  begin
    if lstrcmpi(PChar(ActionName), PChar(FCmdTable[i].Name)) = 0 then
    begin
      Result.cmd := FCmdTable[i].Id;
      Break;
    end;
  end;
end;

function TKeyTable.AccelToStr(const Accel: TAccel): string;
var
  i: Integer;
begin
  Result := '';
  if (Accel.fVirt and FALT) <> 0 then
    Result := 'Alt+';
  if (Accel.fVirt and FSHIFT) <> 0 then
    Result := Result + 'Shift+';
  if (Accel.fVirt and FCONTROL) <> 0 then
    Result := Result + 'Ctrl+';
  for i := 0 to High(SpKeys) do
  begin
    if Accel.key = SpKeys[i].Id then
    begin
      Result := Result + SpKeys[i].Name;
      Exit;
    end;
  end;
  Result := Result + Char(Accel.key);
end;

procedure TKeyTable.SetDefaultKeys(const DefaultKeys: array of string);
var
  i: Integer;
begin
  FKeyListCount := (High(DefaultKeys) + 1) div 2;
  for i := 0 to FKeyListCount - 1 do
    FKeyList[i] := StrToAccel(DefaultKeys[i*2], DefaultKeys[i*2+1]);
end;

procedure TKeyTable.AddMenuItem(Id: Integer);
begin
  if IsMenu(MenuHandle) then
    AppendMenu(MenuHandle, MF_STRING, Id, PChar(IdToKeyNames(Id)));
end;

procedure TKeyTable.GenerateMenu(const MenuTable: array of Integer);
var
  i: Integer;
begin
  for i := 0 to High(MenuTable) do
  begin
    if MenuTable[i] = -1 then
      AppendMenu(MenuHandle, MF_SEPARATOR, 0, nil)
    else
      AddMenuITem(MenuTable[i]);
  end;
end;



end.
